/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.ws.client;

import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.xml.stream.FactoryConfigurationError;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamReader;

import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPBody;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axiom.soap.SOAPHeader;
import org.apache.axiom.soap.impl.builder.StAXSOAPModelBuilder;
import org.objectweb.petals.ws.SoapConstants;
import org.objectweb.petals.ws.addressing.EndpointReference;
import org.objectweb.petals.ws.addressing.WsaConstants;
import org.objectweb.petals.ws.util.AxiomHelper;

/**
 * @author chamerling - eBM WebSourcing
 *
 */
public class SimpleSoapClient implements SoapClient {

    /**
     * Creates a new instance of {@link SimpleSoapClient}
     *
     */
    public SimpleSoapClient() {
        // TODO Auto-generated constructor stub
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.binding.soap.ws.client.SoapClient#send(org.objectweb.petals.binding.soap.ws.addressing.EndpointReference,
     *      org.objectweb.petals.binding.soap.ws.addressing.EndpointReference,
     *      java.lang.String, org.apache.axiom.om.OMElement,
     *      org.apache.axiom.om.OMElement[])
     */
    public SOAPEnvelope send(EndpointReference src, EndpointReference dest,
            String wsaAction, OMElement body, OMElement[] extraHeaders) {
        SOAPEnvelope soapResponse = null;
        SOAPEnvelope soapRequest = createSOAPMessage(src, dest, wsaAction,
                body, extraHeaders);

        OutputStream output = null;
        InputStream is = null;

        try {
            String soapString = soapRequest.toString();
            URL url = new URL(dest.getAddress().toString());
            HttpURLConnection connection = (HttpURLConnection) url
                    .openConnection();
            connection.setRequestMethod("POST");
            connection.setRequestProperty("Content-type",
                    "application/soap+xml");
            connection.setDoOutput(true);
            connection.connect();

            //
            // send the SOAP request...
            //
            output = connection.getOutputStream();
            output.write(soapString.getBytes("UTF-8"));
            output.flush();

            if (connection.getResponseCode() == HttpURLConnection.HTTP_OK) {
                is = connection.getInputStream();
            } else {
                is = connection.getErrorStream();
            }

            soapResponse = getSOAPResponse(is);
            connection.disconnect();
        } catch (Throwable e) {
            soapResponse = createSOAPFault();
        } finally {
            if (output != null) {
                try {
                    output.close();
                } catch (Exception e) {
                }
            }
            if (is != null) {
                try {
                    is.close();
                } catch (Exception e) {
                }
            }
        }
        return soapResponse;
    }

    /**
     * Create a SOAP message from the given data
     *
     * @param sourceEPR
     * @param destinatioNEPR
     * @param wsaAction
     * @param body
     * @param extraHeaders
     * @return
     */
    protected SOAPEnvelope createSOAPMessage(EndpointReference sourceEPR,
            EndpointReference destinatioNEPR, String wsaAction, OMElement body,
            OMElement[] extraHeaders) {

        SOAPEnvelope soapEnvelope = AxiomHelper.createSOAPEnvelope();
        SOAPBody soapBody = AxiomHelper.createSOAPBody(soapEnvelope);
        SOAPHeader soapHeader = AxiomHelper.createSOAPHeader(soapEnvelope);

        // create header
        OMElement toElement = AxiomHelper
                .createOMElement(WsaConstants.TO_QNAME);
        toElement.setText(destinatioNEPR.getAddress().toString());

        OMElement fromElement = AxiomHelper
                .createOMElement(WsaConstants.FROM_QNAME);
        fromElement.setText(sourceEPR.getAddress().toString());

        OMElement messageIDElement = AxiomHelper
                .createOMElement(WsaConstants.MESSAGE_ID_QNAME);

        OMElement actionElement = AxiomHelper
                .createOMElement(WsaConstants.ACTION_QNAME);
        actionElement.setText(wsaAction);

        soapHeader.addChild(toElement);
        soapHeader.addChild(fromElement);
        soapHeader.addChild(messageIDElement);
        soapHeader.addChild(actionElement);

        // Add payload
        soapBody.addChild(body);

        return soapEnvelope;
    }

    /**
     * Get the soap response
     *
     * @param is
     * @return
     */
    protected SOAPEnvelope getSOAPResponse(InputStream is) {
        SOAPEnvelope result = null;
        try {
            XMLStreamReader parser = XMLInputFactory.newInstance()
                    .createXMLStreamReader(is);
            StAXSOAPModelBuilder builder = new StAXSOAPModelBuilder(parser,
                    null);
            // StAXOMBuilder builder = new StAXOMBuilder(factory, parser);
            result = builder.getSOAPEnvelope();
        } catch (XMLStreamException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (FactoryConfigurationError e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return result;
    }

    /**
     *
     * @return
     */
    protected SOAPEnvelope createSOAPFault() {
        SOAPFactory factory = OMAbstractFactory.getSOAP12Factory();
        SOAPEnvelope envelope = factory.createSOAPEnvelope();
        SOAPHeader header = factory.createSOAPHeader(envelope);
        SOAPBody body = factory.createSOAPBody(envelope);
        OMElement faultElement = AxiomHelper
                .createOMElement(SoapConstants.FAULT_QNAME);
        OMElement codeElement = AxiomHelper
                .createOMElement(SoapConstants.CODE_QNAME);
        faultElement.addChild(codeElement);

        body.addChild(faultElement);
        return envelope;
    }

}
