/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.ws.notification;

import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.objectweb.petals.ws.fault.WsnFault;
import org.objectweb.petals.ws.notification.handlers.filter.FilterHandler;
import org.objectweb.petals.ws.notification.handlers.filter.MessageContentFilterHandler;
import org.objectweb.petals.ws.notification.handlers.filter.ProducerPropertiesFilterHandler;
import org.objectweb.petals.ws.notification.handlers.filter.TopicExpressionFilterHandler;
import org.objectweb.petals.ws.topic.WstConstants;

/**
 * A filter factory used to create filter instance from dom elements
 *
 * @author chamerling - eBM WebSourcing
 *
 */
public class FilterFactory {

    /**
     * The factory instance
     */
    private static FilterFactory INSTANCE = null;

    /**
     * Map of filter handlers
     */
    protected Map<QName, FilterHandler> filterHandlers;

    /**
     * Get factory instance
     *
     * @return
     */
    public static FilterFactory getInstance() {
        if (INSTANCE == null) {
            INSTANCE = new FilterFactory();
        }
        return INSTANCE;
    }

    /**
     * Creates a new instance of {@link FilterFactory}
     *
     */
    private FilterFactory() {
        this.filterHandlers = new HashMap<QName, FilterHandler>();
        FilterHandler contentHandler = new MessageContentFilterHandler();
        FilterHandler propertiesHandler = new ProducerPropertiesFilterHandler();
        FilterHandler topicHandler = new TopicExpressionFilterHandler();
        this.filterHandlers
                .put(contentHandler.getHandlerName(), contentHandler);
        this.filterHandlers.put(propertiesHandler.getHandlerName(),
                propertiesHandler);
        this.filterHandlers.put(topicHandler.getHandlerName(), topicHandler);

    }

    /**
     * Get a new filter from the DOM element.
     *
     * @param filterElement
     *            is the <wsnt:Filter> element
     *
     * @return {@link AcceptAllFilter} if no filter element or an instance of
     *         the specified filter.
     */
    public Filter getFilter(OMElement filterElement) throws WsnFault {
        Filter filter = null;

        if (filterElement == null) {
            // filter element is null, the client will accept all the
            // clients messages.
            filter = new AcceptAllFilter();

        } else if (filterElement.getQName().equals(WsnConstants.FILTER_QNAME)) {

            // FIXME = : temporary only support simple topic expression
            OMElement topicElement = filterElement
                    .getFirstChildWithName(WsnConstants.TOPIC_EXPRESSION_QNAME);

            if (topicElement != null) {
                // FIXME : we do not care about namespaces...
                filter = new TopicExpressionFilter(new QName(topicElement
                        .getText()), WstConstants.CONCRETE_TOPIC_URI);
            } else {
                // reject subscription ?
                throw new WsnFault("Can not find a topic expression");
            }

            // FilterList filterList = new FilterList();
            //
            // // create filter for each filter children
            // Iterator children = filterElement.getChildElements();
            // while (children.hasNext()) {
            // Filter tmpFilter = null;
            //
            // OMElement element = (OMElement) children.next();
            // QName filterName = element.getQName();
            // String dialect = element.getAttributeValue(new QName(
            // WsnConstants.DIALECT));
            //
            // // create filter from handler
            // FilterHandler filterHandler = filterHandlers.get(filterName);
            // if (filterHandler != null
            // && filterHandler.accept(filterName, dialect)) {
            // tmpFilter = filterHandler.createNewInstance(filterName,
            // dialect, element.getText());
            // }
            // filterList.addFilter(tmpFilter);
            // }

        } else {
            // FIXME : Really accept all ???
            filter = new AcceptAllFilter();
        }

        // filter should not be null...
        return filter;
    }
}
