/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.ws.notification;

import java.net.URI;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMAttribute;
import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPBody;
import org.objectweb.petals.ws.addressing.EndpointReference;
import org.objectweb.petals.ws.addressing.WsaConstants;
import org.objectweb.petals.ws.fault.WsnFault;
import org.objectweb.petals.ws.topic.WstConstants;
import org.objectweb.petals.ws.util.AxiomHelper;

/**
 * @author chamerling - eBM WebSourcing
 *
 */
public class WsnHelper {

    public static final String XPATH_DIALECT_URI = "http://www.w3.org/TR/1999/REC-xpath-19991116";

    /**
     * Creates a new instance of {@link WsnHelper}
     *
     */
    private WsnHelper() {
    }

    /**
     *
     * @param element
     *            The Filter element
     * @return
     */
    public static Filter filterFromOM(OMElement element) {
        Filter result = null;
        if (element != null) {
            if (!element.getQName().equals(WsnConstants.FILTER_QNAME)) {
                throw new IllegalArgumentException("Not a filter element");
            }
            FilterFactory factory = FilterFactory.getInstance();
            try {
                result = factory.getFilter(element);
            } catch (WsnFault e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }

        } else {
            result = new AcceptAllFilter();
        }
        return result;
    }

    /**
     *
     * @param filter
     * @return
     */
    public static OMElement filterToOM(Filter filter) {
        OMElement result = AxiomHelper
                .createOMElement(WsnConstants.FILTER_QNAME);

        // topic expression
        // String topicExpressionValue = filter.getTopicExpression();
        String topicDialect = WstConstants.CONCRETE_TOPIC_URI;
        OMElement topicElement = AxiomHelper
                .createOMElement(WsnConstants.TOPIC_EXPRESSION_QNAME);
        topicElement.addAttribute(AxiomHelper.createOMAttribute(
                WsnConstants.DIALECT, WsnConstants.FILTER_QNAME, topicDialect));
        topicElement.setText(filter.getTopicName().toString());
        result.addChild(topicElement);

        // producer properties
        // String producerContent = "TODO producer content";
        // String producerDialect = "TODO producer dialect";
        // OMElement producerElement = AxiomHelper
        // .createOMElement(WsnConstants.PRODUCER_PROPERTIES_QNAME);
        // producerElement.addAttribute(AxiomHelper.createOMAttribute(
        // WsnConstants.DIALECT, null, producerDialect));
        // producerElement.setText(producerContent);
        // result.addChild(producerElement);

        // message content
        // String messageContent = "TODO message content";
        // String messageDialect = "TODO message dialect";
        // OMElement messageElement = AxiomHelper
        // .createOMElement(WsnConstants.MESSAGE_CONTENT_QNAME);
        // messageElement.addAttribute(AxiomHelper.createOMAttribute(
        // WsnConstants.DIALECT, null, messageDialect));
        // messageElement.setText(messageContent);
        // result.addChild(messageElement);

        return result;
    }

    /**
     *
     * @param element
     * @return
     */
    public static Date dateFromOM(OMElement element) {
        if (element == null) {
            throw new NullPointerException("Date element is null");
        }
        Date result = new Date();
        // TODO
        return result;
    }

    /**
     * Create a {@link Subscribe} instance from an {@link OMElement}
     *
     * @param element
     * @return
     */
    public static Subscribe subscribeFromOM(OMElement element) {
        if (element == null) {
            throw new NullPointerException("Subcribe element is null");
        }

        OMElement consumerElement = element
                .getFirstChildWithName(WsnConstants.CONSUMER_QNAME);

        OMElement consumerEPRElement = consumerElement
                .getFirstChildWithName(WsaConstants.ADDRESS_QNAME);

        EndpointReference epr = new EndpointReference(URI
                .create(consumerEPRElement.getText()));

        // filter
        OMElement filterElement = element
                .getFirstChildWithName(WsnConstants.FILTER_QNAME);
        Filter filter = null;
        try {
            filter = FilterFactory.getInstance().getFilter(filterElement);
        } catch (WsnFault e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        // date
        Date date = null;
        OMElement dateElement = element
                .getFirstChildWithName(WsnConstants.INIT_TERMINATION_TIME_QNAME);
        if (dateElement != null) {
            date = dateFromOM(dateElement);
        }

        Subscribe result = new Subscribe(epr, filter, date);
        return result;
    }

    /**
     *
     * @param subscribe
     * @return
     */
    public static OMElement subscribeToOM(Subscribe subscribe) {
        OMElement result = AxiomHelper
                .createOMElement(WsnConstants.SUBSCRIBE_QNAME);

        // add consumer reference element
        // OMElement epr = EndpointReferenceHelper.toOM(OMAbstractFactory
        // .getOMFactory(), subscribe.getConsumer(),
        // WsnConstants.CONSUMER_QNAME, WsaConstants.NAMESPACE_URI);

        // consumer reference
        OMElement consumerElement = AxiomHelper
                .createOMElement(WsnConstants.CONSUMER_QNAME);
        OMElement eprElement = AxiomHelper
                .createOMElement(WsaConstants.ADDRESS_QNAME);
        eprElement.setText(subscribe.getConsumerEPR().getAddress().toString());
        consumerElement.addChild(eprElement);
        result.addChild(consumerElement);

        // add filter
        if (subscribe.getFilter() != null) {
            OMElement filterElement = filterToOM(subscribe.getFilter());
            result.addChild(filterElement);
        }

        // add time
        if (subscribe.getTerminationTime() != null) {
            OMElement dateElement = AxiomHelper
                    .createOMElement(WsnConstants.TERMINATION_TIME_QNAME);
            dateElement.setText(subscribe.getTerminationTime().toString());
            result.addChild(dateElement);
        }

        return result;
    }

    /**
     *
     * @param element
     * @return
     */
    public static NotificationMessage notificationMessageFromOM(
            OMElement element) {
        NotificationMessage result = new NotificationMessage();

        OMElement eprElement = element
                .getFirstChildWithName(WsnConstants.SUBSCRIPTION_EPR_QNAME);
        if (eprElement == null) {
            throw new NullPointerException("");
        }

        EndpointReference epr = new EndpointReference(URI.create(eprElement
                .getText()));
        result.setSubscriptionEPR(epr);

        // String currentString = XmlUtils.getElementText(xml,
        // WsnConstants.CURRENT_TIME_QNAME);
        // String terminationString = XmlUtils.getElementText(xml,
        // WsnConstants.TERMINATION_TIME_QNAME);

        // try {
        // if (currentString != null)
        // _currentTime = XsdUtils.getLocalTime(currentString);
        //
        // if (terminationString != null)
        // _terminationTime = XsdUtils.getLocalTime(terminationString);
        // }
        //
        // catch (ParseException error) {
        // throw new InvalidMessageFormatFault(error);
        // }

        return result;
    }

    /**
     *
     * @param message
     * @return
     */
    public static OMElement notificationMessageToOM(NotificationMessage message) {
        OMElement result = AxiomHelper
                .createOMElement(WsnConstants.NOTIFICATION_MSG_QNAME);

        // subscription EPR
        EndpointReference subscriptionEPR = message.getSubscriptionEPR();
        if (subscriptionEPR != null) {
            OMElement eprElement = AxiomHelper
                    .createOMElement(WsaConstants.ADDRESS_QNAME);
            eprElement.setText(subscriptionEPR.getAddress().toString());
            OMElement subscriptionElement = AxiomHelper
                    .createOMElement(WsnConstants.SUBSCRIPTION_EPR_QNAME);
            subscriptionElement.addChild(eprElement);
            result.addChild(subscriptionElement);
        }

        // topic
        QName topicPath = message.getTopic();
        if (topicPath != null) {
            OMElement topicElement = AxiomHelper
                    .createOMElement(WsnConstants.TOPIC_QNAME);
            topicElement.setText(topicPath.toString());
            topicElement.addAttribute(WsnConstants.DIALECT, message
                    .getTopicDialect(), null);
            result.addChild(topicElement);
        }

        // producer EPR
        EndpointReference producerEPR = message.getProducerEPR();
        if (producerEPR != null) {
            OMElement eprElement = AxiomHelper
                    .createOMElement(WsaConstants.ADDRESS_QNAME);
            eprElement.setText(producerEPR.getAddress().toString());
            OMElement producerElement = AxiomHelper
                    .createOMElement(WsnConstants.PRODUCER_QNAME);
            producerElement.addChild(eprElement);
            result.addChild(producerElement);
        }

        // Message
        OMElement messageElement = AxiomHelper
                .createOMElement(WsnConstants.MESSAGE_QNAME);
        result.addChild(messageElement);

        Map<QName, OMElement> messageContent = message.getMessageContent();
        Iterator<QName> iter = messageContent.keySet().iterator();
        while (iter.hasNext()) {
            QName qname = iter.next();
            OMElement contentElement = messageContent.get(qname);
            messageElement.addChild(contentElement);
        }

        // TODO : add message NS to root ...

        return result;
    }

    /**
     * Create an OMElement from the subscribe response object.
     *
     * @param response
     * @return a OMElement representation of the subscribe response :
     *
     * <pre>
     *           &lt;wsnt:SubscribeResponse&gt;
     *             &lt;wsnt:SubscriptionReference&gt;
     *               wsa:EndpointRefenceType
     *             &lt;/wsnt:SubscriptionReference&gt;
     *             ...
     *           &lt;/wsnt:SubscribeResponse&gt;
     * </pre>
     */
    public static OMElement subscribeResponseToOM(SubscribeResponse response) {
        OMElement result = AxiomHelper
                .createOMElement(WsnConstants.SUBSCRIBE_RESPONSE_QNAME);

        // EPR
        EndpointReference epr = response.getSubscriptionReference();
        OMElement subEprElement = AxiomHelper
                .createOMElement(WsnConstants.SUBSCRIPTION_EPR_QNAME);
        OMElement eprElement = AxiomHelper
                .createOMElement(WsaConstants.ADDRESS_QNAME);
        if (eprElement != null) {
            eprElement.setText(epr.getAddress().toString());
            subEprElement.addChild(eprElement);
        }
        result.addChild(subEprElement);

        // Date
        String currentTime = response.getCurrentTime();
        String termTime = response.getTerminationTime();

        if (currentTime != null) {
            OMElement currentTimeElement = AxiomHelper
                    .createOMElement(WsnConstants.CURRENT_TIME_QNAME);
            currentTimeElement.setText(currentTime);
            result.addChild(currentTimeElement);
        }

        if (termTime != null) {
            OMElement termTimeElement = AxiomHelper
                    .createOMElement(WsnConstants.TERMINATION_TIME_QNAME);
            termTimeElement.setText(termTime);
            result.addChild(termTimeElement);
        }
        return result;
    }

    /**
     *
     * @param soapBody
     * @return
     */
    public static SubscribeResponse subscribeResponseFromOM(SOAPBody soapBody)
            throws WsnFault {
        if (soapBody == null) {
            throw new WsnFault("SubscribeResponse is null");
        }
        SubscribeResponse result = new SubscribeResponse();

        // subscription reference
        OMElement subscribeResponseElement = soapBody
                .getFirstChildWithName(WsnConstants.SUBSCRIBE_RESPONSE_QNAME);

        OMElement eprElement = subscribeResponseElement
                .getFirstChildWithName(WsnConstants.SUBSCRIPTION_EPR_QNAME);
        EndpointReference epr = new EndpointReference(URI.create(eprElement
                .getText()));
        result.setSubscriptionEPR(epr);

        // Time
        OMElement timeElement = subscribeResponseElement
                .getFirstChildWithName(WsnConstants.CURRENT_TIME_QNAME);
        if (timeElement != null) {
            result.setCurrentTime(timeElement.getText());
        }
        OMElement terminationElement = subscribeResponseElement
                .getFirstChildWithName(WsnConstants.TERMINATION_TIME_QNAME);
        if (terminationElement != null) {
            result.setTerminationTime(terminationElement.getText());
        }
        return result;
    }

    /**
     *
     * @param current
     * @return
     */
    public static OMElement getCurrentMessageToOM(GetCurrentMessage current) {
        OMElement result = AxiomHelper
                .createOMElement(WsnConstants.GET_CURRENT_QNAME);
        OMElement topicPathElement = AxiomHelper
                .createOMElement(WsnConstants.TOPIC_QNAME);
        topicPathElement.setText(current.getTopicPath());

        String dialect = current.getDialect(current.getTopicPath()
                .getLocalPart());
        OMAttribute omAttribute = AxiomHelper.createOMAttribute(
                WsnConstants.DIALECT, WsnConstants.TOPIC_QNAME, dialect);
        topicPathElement.addAttribute(omAttribute);
        result.addChild(topicPathElement);
        return result;
    }

    /**
     * Get the current message object from OMElement
     *
     * @param element
     * @return
     */
    public static GetCurrentMessage getCurrentMessageFromOM(OMElement element)
            throws WsnFault {
        if (element == null) {
            throw new WsnFault("Can not parse a null element");
        }
        if (element.getQName() != WsnConstants.GET_CURRENT_QNAME) {
            throw new WsnFault("Invalid node : " + element.getQName());
        }
        GetCurrentMessage result = null;

        OMElement topicElement = element
                .getFirstChildWithName(WstConstants.TOPIC_QNAME);
        if (topicElement != null) {
            result = new GetCurrentMessage(topicElement.getTextAsQName());
        }
        return result;
    }

    /**
     *
     * @param element
     * @return
     */
    public static GetCurrentMessageResponse getCurrentMessageResponseFromOM(
            OMElement element) {
        GetCurrentMessageResponse result = null;

        OMElement currentElement = element
                .getFirstChildWithName(WsnConstants.GET_CURRENT_QNAME);
        OMElement notifElement = currentElement
                .getFirstChildWithName(WsnConstants.NOTIFICATION_MSG_QNAME);

        NotificationMessage notificationMessage = WsnHelper
                .notificationMessageFromOM(notifElement);
        result = new GetCurrentMessageResponse(notificationMessage);

        return result;
    }
}
